/**
  @enum {string}
 */
PouchDB.Adapters = {
  LevelDB: 'levelDB'
};

/**
  adapter - 'idb', 'leveldb', 'websql', or 'http'
  @typedef {{
    name: (string|undefined),
    auto_compaction: (boolean|undefined),
    adapter: (string|undefined),
    ajax: (Object|undefined)
  }}
 */
var PouchDBConstructorOptions;

/**
  @typedef {{
    _id: (string),
    _rev: (string|undefined)
  }}
 */
var PouchDBDocument;

/**
 @typedef {{
    ok: boolean,
    id: string,
    rev: string
  }}
 */
var PouchDBResponse;

/**
  1) error
  2) info
  @typedef {function(Object, PouchDBResponse)}
 */
var PouchDBCallback;

/**
 @typedef {{
    doc: Object,
    key: string,
    value: string,
    id: string
  }}
 */
var PouchDBAllDocsDocument;

/**
  @typedef {{
    offset: number,
    rows: Array.<PouchDBAllDocsDocument>,
    total_rows: number
  }}
 */
var PouchDBAllDocsResult;

/**
  @constructor
  @param {string=} name
  @param {PouchDBConstructorOptions=} options
 */
var PouchDB = function(name, options) {};

/**
  @param {Object} doc
  @param {string=} docId
  @param {string=} docRev
  @param {Object=} options
  @param {Function=} callback
 */
PouchDB.prototype.put = function(doc, docId, docRev, options, callback) {};


/**
 @param {Object|string} doc - object or id of the object
 @param {PouchDBCallback=} callback
 */
PouchDB.prototype.get = function(doc, callback) {};

PouchDB.prototype.destroy = function(options, callback) {};

PouchDB.prototype.replicate = {

  /**
    @param {string} uri
   */
  to: function(uri) {},

  /**
    @param {string} uri
   */
  from: function(uri) {}
};

PouchDB.prototype.changes = function() {
  return {

    /**
      @param {string} event
      @param {PouchDBCallback} callback
     */
    on: function(event, callback) {}
  };
};

/**
 * @param {Object} options
 * @return {PouchDBAllDocsResult | Promise}
 */
PouchDB.prototype.allDocs = function(options) {};


/**
 * @param {Object} doc
 * @return {Promise}
 */
PouchDB.prototype.remove = function(doc) {};

