goog.require 'app.editor.Block'
goog.require 'app.BlockType'
goog.require 'app.DocumentStore'
goog.require 'app.document.Document'
goog.require 'app.document.Block'
goog.require 'tests.Factory'

suite 'app.DocumentStore', ->

  createBlock = tests.Factory.Block.create
  createDocument = tests.Factory.Document.createWithOpts

  setup ->
    [@store, @storageMock] = tests.Factory.DocumentStore.createAndMockStorage()

  test 'init ok', ->
    assert.ok @store

  test 'getElementBlockTypes', ->
    types = @store.getElementBlockTypes()
    assert.isArray types

  suite 'createBlock', ->

    test 'object literal', ->
      block = {}
      validated = @store.createBlock(block)
      assert.instanceOf validated, app.document.Block
      assert.ok block != validated, 'created new object from object literal'

    test 'existing Block object', ->
      block = createBlock()
      validated = @store.createBlock(block)
      assert.instanceOf validated, app.document.Block
      assert.ok block == validated, 'no validation needed'

    test 'object literal with blocks property', ->
      block = {blocks: [{}, {}]}
      validated = @store.createBlock block
      assert.instanceOf validated, app.document.StructuredBlock
      nestedBlock = validated.blocks[0]
      assert.instanceOf nestedBlock, app.document.Block, 'creates blocks recursively'

  suite 'logic getters', ->
    test 'can nest', ->
      # 5
      dArt = {title: "Musée d'Art Moderne de la Ville de Paris", type: app.BlockType.Section}
      # 4
      museums = {title: "Paris museums", blocks: [dArt], type: app.BlockType.Section}
      # 1
      @store.document = {
        type: app.BlockType.Document
        blocks: [
          # 2
          {
            type: app.BlockType.Section
            blocks: [
              # 3
              museums
            ]
          }
        ]
      }
      assert.ok @store.canNestStructured museums
      assert.notOk @store.canNestStructured dArt

  suite 'document parts getters', ->
    setup ->
      @paragraph = {type: app.BlockType.Paragraph, text: 'deep paragraph'}
      @store.document = createDocument {
        blocks: [
          {
            type: app.BlockType.Section
            blocks: [
              @paragraph
              {type: app.BlockType.Paragraph}
              {type: app.BlockType.Paragraph}
            ]
          }
          {
            type: app.BlockType.List, items: ['top', 'level', 'list']
          }
          {type: app.BlockType.Paragraph}
        ]
      }

    test 'finds top level block', ->
      list = @store.findFirstOfType app.BlockType.List
      assert.deepEqual list.items, ['top', 'level', 'list']

    test 'finds block nested in structure', ->
      result = @store.findFirstOfType app.BlockType.Paragraph
      assert.equal result.text, 'deep paragraph'

    test 'finds structured block', ->
      section = @store.findFirstOfType app.BlockType.Section
      assert.equal section.blocks[0].text, 'deep paragraph'

    test 'findOfType', ->
      allParagraphs = @store.findOfType app.BlockType.Paragraph, @store.document
      assert.lengthOf allParagraphs, 4

  suite 'create and read', ->
    setup ->
      @opera = createBlock {title: 'The Opéra Garnier'}
      @cinema = createBlock {title: 'Le Grand Rex', text: 'Paris largest cinema with 2,800 seats.'}
      @parisCulture = createBlock {
        title: 'Paris culture'
        type: app.BlockType.Section
        blocks: [
          @opera
          @cinema
        ]
      }
      @document = createDocument {
        title: 'Document'
        blocks: [
          createBlock {title: 'Rome culture'}
          @parisCulture
        ]
      }
      @store.document = @document

    test 'findBlock not found', ->
      found = @store.findBlockParent({title: 'The Opéra Garnier'})
      assert.isNull found, 'returns null if not found'

    test 'findBlock should find', ->
      found = @store.findBlockParent(@cinema)
      assert.ok found, 'should found'
      assert.deepEqual found, @parisCulture
      assert.equal found, @parisCulture, 'finds by reference'