goog.provide 'app.editor.Block'

###
  Fileoverview
  Block class is a logic block.
  Suggested names: TextBlock, SimpleBlock
###
goog.require 'app.ui.ActivableBlock'
goog.require 'app.ui.ShortcutsMixin'
goog.require 'app.BlockType'
goog.require 'app.editor.Paragraph'
goog.require 'app.editor.List'
goog.require 'app.ui.RemoveButton'
#goog.require 'app.editor.MetaPanel'

class app.editor.Block
  ###*
    @param {app.EditorStore} editorStore
    @param {app.Routes} routes
    @param {app.editor.MetaPanel} metaPaneModule
    @param {app.ui.ShortcutsMixin} shortcuts
    @constructor
  ###
  constructor: (editorStore, routes, metaPaneModule, shortcuts) ->
    {a, div, input, option, select} = React.DOM

    Paragraph = app.editor.Paragraph.reactClass
    list = app.editor.List.reactClass
    MetaPanel = metaPaneModule.reactClass

    @reactClass = React.createClass(
      displayName: 'block'

#      mixins: [React.addons.PureRenderMixin]
      mixins: [shortcuts.mixin, app.ui.ActivableBlock]

      # TODO: remove event
      onTitleChange: (block, e) ->
        editorStore.changeTitle block, e.target.value
        @forceUpdate()

    # TODO: remove event
      onTypeChange: (block, e) ->
        console.log 'type change' if app.lamebug
        editorStore.changeType block, e.target.value

      ###*
        @param {app.document.Block} block
        @param {string} newComment
      ###
      onCommentChange: (block, newComment) ->
        console.log 'comment change' if app.lamebug
        editorStore.changeComment block, newComment
        @forceUpdate()


      ###*
        @param {app.document.Block} block
        @param {string} newNote
      ###
      onNoteChange: (block, newNote) ->
        console.log 'note change' if app.lamebug
        editorStore.changeNote block, newNote
        @forceUpdate()

      render: ->
        editorProps = {
          id: @props.block._id
          onEditorChange: @props.onEditorChange.bind(null, @props.block)
          ref: 'editor'
        }
        editor = switch @props.block.type
          when app.BlockType.Paragraph
            editorProps.text = @props.block.text
            Paragraph(editorProps)
          when app.BlockType.List
            editorProps.items = @props.block.items
            list(editorProps)
          else
            Paragraph(editorProps)
        content = div({
          className: 'content'
        }, editor)
        maybeActive = if @props.isActive then 'block-active' else ''
        return div(
          {
            className: "block #{maybeActive}"
            onKeyDown: @onShortcut
            onClick: @onClick
            key: @props.block._id
          }
          content, @renderMeta()
        )

      onListTypeChange: (e) ->
        editorStore.changeListType(@props.block, e.target.value)

      listOptions: ->
        return unless @props.block.type == app.BlockType.List
        type = if @props.block.isOrdered then 'ordered' else 'unordered'
        options = ['ordered', 'unordered'].map (type) -> option({key: type}, type)
        return div(
          {}
          'List type: '
          select(
            {value: type, onChange: @onListTypeChange}
            options
          )
        )

      renderMeta: ->
        title = div(
          {className: 'title'}
          input({
            className: 'title'
            placeholder: 'block title'
            value: @props.block.title
            ref: 'title'
            onChange: @onTitleChange.bind(this, @props.block)
          })
        )
        return div(
          {
            className: 'meta'
          }
          title
          app.ui.RemoveButton(
            {
              onRemove: editorStore.deleteBlock.bind(null, @props.block)
              title: 'delete'
              classSuffix: 'block'
            }
          )
          MetaPanel(
            {
              block: @props.block
              types: @props.types
              typeOptions: @listOptions()
              onTypeChange: @onTypeChange.bind(@, @props.block)
              onCommentChange: @onCommentChange
              onNoteChange: @onNoteChange
              blockViewRoute: routes.view.url({blockId: @props.block._id})
            }
          )
        )

    )
