goog.provide 'app.editor.TransformType'

goog.require 'app.BlockType'
goog.require 'app.document.List'
goog.require 'app.document.Paragraph'

class app.editor.TransformType

  ###*
    @constructor
  ###
  constructor: ->

  ###*
    @public
    @param block {app.document.Block}
    @param newType {app.BlockType}
    @returns {app.document.Block} block with type given by newType
  ###
  transformTo: (block, newType) ->
    console.log 'Changing type of object: ', block if app.lamebug
    transformFun = @_getTransformFunction(block.type, newType)
    throw Error 'no transformFun' unless (transformFun)
    return transformFun(block)

  ###*
    @param oldType {app.BlockType}
    @param newType {app.BlockType}
  ###
  _getTransformFunction: (oldType, newType) ->
    blockTypes = app.BlockType.getAll()
    throw Error "Unsupported from type: #{oldType}" if blockTypes.indexOf(oldType) == -1
    throw Error "Unsupported to type: #{newType}" if blockTypes.indexOf(newType) == -1
    unless (@transform)
      @transform = {}
      fromParagraph = @transform[app.BlockType.Paragraph] = {}
      fromList = @transform[app.BlockType.List] = {}

      fromParagraph[app.BlockType.List] = @_paragraph_to_list
      fromList[app.BlockType.Paragraph] = @_list_to_paragraph
    return @transform[oldType][newType]

  ###*
    @param list {app.document.List}
  ###
  _list_to_paragraph: (list) ->
    unless list.items?
      throw Error "Block #{list.constructor.name} should have 'items' property"

    paragraph = new app.document.Paragraph(list)
    paragraph.text = list.items.join('\n') || ''
    delete list.items
    list = null
    return paragraph

  ###*
    @param paragraph {app.document.Paragraph}
  ###
  # OWN class MODULE
  _paragraph_to_list: (paragraph) ->
    unless paragraph.text?
      throw Error "Block #{paragraph.constructor.name} should have 'text' property"

    list = new app.document.List(paragraph)
    list.items = paragraph.text.split('\n') || []
    delete list.text
    paragraph = null
    return list
