goog.provide 'app.ui.ShortcutsMixin'

goog.require 'app.BlockType'
goog.require 'goog.events.KeyCodes'
goog.require 'goog.ui.KeyboardShortcutHandler'
goog.require 'goog.events'

class app.ui.ShortcutsMixin

  ###*
    @param {app.EditorStore} editorStore
    @constructor
  ###
  constructor: (editorStore) ->
    @mixin = {

      onClick: (e) ->
        e.stopPropagation()
        editorStore.activateBlockIfNeeded @props.block

      onShortcut: (e) ->
        if e.ctrlKey
          if e.altKey
            @handleCtrlAndAltKey(e)
          else
            @handleCtrlKey(e)
        else if e.altKey
          @handleAltKey(e)

      handleAltKey: (e) ->
        block = @props.block
        switch e.keyCode
          when goog.events.KeyCodes.W
            @useShortcut(e)
            editorStore.moveUP(block)
          when goog.events.KeyCodes.S
            @useShortcut(e)
            editorStore.moveDOWN(block)
          when goog.events.KeyCodes.A
            @useShortcut(e)
            editorStore.moveLEFT(block)
          when goog.events.KeyCodes.D
            @useShortcut(e)
            editorStore.moveRIGHT(block)
          when goog.events.KeyCodes.T
            @useShortcut(e)
            @toggleFocus()

      handleCtrlKey: (e) ->
        switch e.keyCode
          when goog.events.KeyCodes.K
            @useShortcut(e)
            editorStore.activateUpTo @props.block
          when goog.events.KeyCodes.J
            @useShortcut(e)
            editorStore.activateDownTo @props.block
          when goog.events.KeyCodes.H
            @useShortcut(e)
            editorStore.activateOutsideTo @props.block
          when goog.events.KeyCodes.L
            @useShortcut(e)
            editorStore.activateInsideTo @props.block

      handleCtrlAndAltKey: (e) ->
        switch e.keyCode
          when goog.events.KeyCodes.P
            @useShortcut(e)
            editorStore.addBlockAfter(@props.block, app.BlockType.Paragraph)
          when goog.events.KeyCodes.I
            @useShortcut(e)
            editorStore.addBlockAfter(@props.block, app.BlockType.Image)
          when goog.events.KeyCodes.O
            @useShortcut(e)
            editorStore.addBlockAfter(@props.block, app.BlockType.Section)

      useShortcut: (e) ->
        e.preventDefault()
        e.stopPropagation()


      toggleFocus: ->
        # section does not have one
        if @refs.editor? && document.activeElement == @refs.editor.getDOMNode()
          @refs.title.getDOMNode().focus()
        else
          @refs.editor.getDOMNode().focus()
    }