goog.provide 'devel.seed'

goog.require 'app.BlockType'

devel.seed = {"type":"document","__id":"q6qtbzr7xm35","tags":[],"blocks":[{"type":"section","blocks":[{"type":"paragraph","tags":[],"text":"While HTML is used to define the structure and content of a web page and CSS encodes the style of how the formatted content should be graphically displayed, JavaScript is used to add interactivity to a web page or create rich web applications.\n\nHowever, the umbrella term \"JavaScript\" as understood in the web browser context contains several very different elements. One of them is the core language (ECMAScript), another is the collection of the Web APIs, including the DOM (Document Object Model).","_id":null}],"tags":[],"title":"Introduction","_id":null},{"type":"section","blocks":[{"type":"paragraph","tags":[],"text":"The core language of JavaScript is standardized by the ECMA TC39 committee as a language named ECMAScript. The latest version of the specification is ECMAScript 5.1.\n\nThis core language is also used in non-browser environments, for example in node.js.","_id":null},{"type":"section","blocks":[{"type":"paragraph","tags":[],"text":"Among other things, ECMAScript defines:","_id":null},{"type":"list","tags":[],"items":["The language syntax (parsing rules, keywords, control flow, object literal initialization...)","Error handling mechanisms (throw, try/catch, ability to create user-defined Error types)","Types (boolean, number, string, function, object...)","The global object. In a browser environment, this global object is the window object, but ECMAScript only defines the APIs, not specific to browsers, e.g. parseInt, parseFloat, decodeURI, encodeURI...","A prototype-based inheritance mechanism","Built-in objects and functions (JSON, Math, Array.prototype methods, Object introspection methods...)","Strict mode"],"_id":null}],"tags":[],"title":"What falls under the ECMAScript scope?","_id":null},{"type":"section","blocks":[{"type":"paragraph","tags":[],"text":"As of August 2014, the current versions of the main web browsers implement ECMAScript 5.1, but older versions are still in use that implement ECMAScript 3 and only parts of ECMAScript 5. Modern Browsers already implement large parts of ECMAScript 6.","_id":null}],"tags":[],"title":"Browser support","_id":null},{"type":"section","blocks":[{"type":"paragraph","tags":[],"text":"The proposed fourth edition of ECMA-262 (ECMAScript 4 or ES4) would have been the first major update to ECMAScript since the third edition was published in 1999. As of August 2008, the ECMAScript 4th edition proposal has been scaled back into a project codenamed ECMAScript Harmony, that defines among others, things like Proxies or the const keyword. Progress can be followed here. A major 6th Edition of the standard is expected to be released in 2015.","_id":null}],"tags":[],"title":"Future","_id":null}],"tags":[],"title":"JavaScript, the core language (ECMAScript)","_id":null},{"type":"section","blocks":[{"type":"paragraph","tags":[],"text":"The ECMAScript Internationalization API Specification is an addition to the ECMAScript Language Specification, also standardized by Ecma TC39. The internationalization API prov_ides collation (string comparison), number formatting, and date and time formatting for JavaScript applications, and lets applications choose the language and tailor the functionality to their needs. The standard was approved in December 2012; the status of implementations in browsers is tracked in the documentation of the Intl object.","_id":null}],"tags":[],"title":"Internationalization API","_id":null},{"type":"section","blocks":[{"type":"paragraph","tags":[],"title":"WebIDL","text":"The WebIDL specification prov_ides the glue between the DOM technologies and ECMAScript.","_id":null},{"type":"section","blocks":[{"type":"paragraph","tags":[],"text":"The Document Object Model (DOM) is a cross-platform and language-independent convention for representing and interacting with objects in HTML, XHTML and XML documents. Objects in the DOM tree may be addressed and manipulated by using methods on the objects. The Core Document Object Model is standardized by the W3C. It defines language-agnostic interfaces which abstract HTML and XML documents as objects and mechanisms to manipulate this abstraction. Among the things defined by the DOM, we can find:","_id":null},{"type":"list","tags":[],"items":["The document structure, a tree model, and the DOM Event architecture in DOM core: Node, Element, DocumentFragment, Document, DOMImplementation, Event, EventTarget, \u2026","A less rigorous definition of the DOM Event Architecture, as well as specific events in DOM events.","Other things such as DOM Traversal and DOM Range."],"_id":null},{"type":"paragraph","tags":[],"text":"From the ECMAScript point of view, objects defined in the DOM specification are called \"host objects\".\n","_id":null}],"tags":[],"title":"The Core of the DOM","_id":null},{"type":"section","blocks":[{"type":"paragraph","tags":[],"text":"HTML, the Web's markup language, is specified in terms of the DOM. Layered above the abstract concepts defined in DOM Core, HTML also defines the meaning of elements. The HTML DOM includes such things as the className property on HTML elements, or APIs such as document.body.\n\nThe HTML specification also defines restrictions on documents; for example, it requires all children of a ul element, which represents an unordered list, to be li elements, as those represent list items. In general, it also forb_ids using elements and attributes that aren't defined in a standard.\n\nLooking for the Document object, Window object, and the other DOM elements? Read the DOM documentation.","_id":null}],"tags":[],"title":"HTML DOM","_id":null},{"type":"list","tags":[],"title":"Other notable APIs","items":["The setTimeout and setInterval functions have been first specified on the Window interface in HTML Standard.","XMLHttpRequest. API allowing to send asynchronous HTTP request.","CSS Object Model. The CSSOM is used to abstract CSS rules as objects","WebWorkers. API that allows parallel computation.","WebSockets. API that allows low-level b_idirectional communication.","Canvas 2D Context. Drawing API for the canvas element."],"_id":null},{"type":"paragraph","tags":[],"title":"Browser support","text":"Every web developer has experienced that the DOM is a mess. Browser support uniformity varies a lot from feature to feature. The main reason for this situation is the fact that many important DOM features have had very unclear, if any, specifications. Also, different web browsers have added incompatible features for overlapping use cases (like the Internet Explorer event model). The current (as of June 2011) trend is that the W3C and particularly the WHATWG are defining older features in detail, in order to improve interoperability. Following this trend, browsers are improving their implementations based on these specifications.\n\nOne common, though perhaps not the most reliable, approach to cross-browser compatibility is to use a JavaScript library. These libraries abstract DOM features and ensure their APIs work similarly in different browsers. Some of the most w_idely used frameworks are jQuery, prototype, and YUI.","_id":null}],"tags":[],"title":"The Web APIs and the DOM","_id":null}],"title":"JavaScript technologies overview"}